/*
* This program is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License as published by
* the Free Software Foundation; either version 2 of the License, or
* (at your option) any later version.

* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
* GNU General Public License for more details.

* You should have received a copy of the GNU General Public License
* along with this program; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*
* Copyright (C) 2009 - Marcelo Leal.
*/


#define _GNU_SOURCE
#define IMENU 11
#define NBLOCKS 9
#define TAM512 512 
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <regex.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <menu.h>
#include <curses.h>

#define ARRAY_SIZE(a) (sizeof(a) / sizeof(a[0]))
#define CTRLD 4

#define _CONF_INTEGER   1
#define _CONF_STRING    2
#define CONF_INTEGER(x, y)              { x, &y, _CONF_INTEGER, 0 }
#define CONF_NULL                       { NULL, NULL, 0, 0 }
#define CONF_STRPTR_INTEGER(w, z)       CONF_INTEGER(#z, w->z)


struct dtraceoutput {
    int r;
    int w;
};
struct blockdbfields {
    char *op_name;
    int *op_variable;
    int op_type;
    int op_maxlength;
};

char *blocksizes[] = {
			"-= Block  Sizes =-: ",
			"                    ",
			"      <=1KB       : ",
			">1KB   ...   <=2KB: ",
			">2KB   ...   <=4KB: ",
			">4KB   ...   <=8KB: ",
			">8KB   ...  <=16KB: ",
			">16KB  ...  <=32KB: ",
			">32KB  ...  <=64KB: ",
			">64KB  ... <=128KB: ",
                        "     >128KB       : ",
			(char *)NULL,
		};

void print_str_middle(WINDOW *win, int starty, int startx, int width, char *string, chtype color);
void print_int_middle(WINDOW *win, int starty, int startx, int width, int *inteiro, chtype color);
static int read_draw_data(struct dtraceoutput *pdtrace, WINDOW *win);

void print_str_middle(WINDOW *win, int starty, int startx, int width, char *string, chtype color) {
   int length, x, y;
   float temp;
   
   if(win == NULL)
   win = stdscr;
   getyx(win, y, x);
   if(startx != 0)
   x = startx;
   if(starty != 0)
   y = starty;
   if(width == 0)
   width = 80;

   length = strlen(string);
   temp = (width - length)/ 2;
   x = startx + (int)temp;
   wattron(win, color);
   mvwprintw(win, y, x, "%s", string);
   wattroff(win, color);
   refresh();
}

void print_int_middle(WINDOW *win, int starty, int startx, int width, int *inteiro, chtype color) {
   int length, x, y;
   if(win == NULL)
   win = stdscr;
   getyx(win, y, x);
   if(startx != 0)
   x = startx;
   if(starty != 0)
   y = starty;
   wattron(win, 2);
   mvwprintw(win, x, y, "%8lu", *inteiro);
   wattroff(win, color);
   refresh();
}


// Read data...
static int read_draw_data(struct dtraceoutput *pdtrace, WINDOW *win) {
    FILE *f;
    struct blockdbfields blockdb[] = {
        CONF_STRPTR_INTEGER(pdtrace, r),
        CONF_STRPTR_INTEGER(pdtrace, w),
        CONF_NULL,
    };
    struct blockdbfields *p;
    char buffer[TAM512];
    char *vname, *end, *value, *temp=NULL, *errorptr;
    int i, intvalue, ln, length;
    int *gp, *gpt, coord=0;

    // Where we will hold the data
    int rvalues[10], wvalues[10], trvalues[10], twvalues[10]; 
    int *prvalues = rvalues, *pwvalues = wvalues; 
    int *ptrvalues = trvalues, *ptwvalues = twvalues;

    // We have this in some place, but i don't know where... ;)
    int v1k=1024, v2k=2048, v4k=4096, v8k=8192; 
    int v16k=16384, v32k=32768, v64k=65536, v128k=131072;

    for (i = 0; i < NBLOCKS; i++)
    {
      prvalues[i] = 0; pwvalues[i] = 0; ptrvalues[i] = 0; ptwvalues[i] = 0;
    }

    ln = 0;
    while (fgets(buffer, sizeof(buffer), stdin)) {
        ln++;
        length = strlen(buffer);
        if (buffer[length - 1] != '\n') {
            while (fgets(buffer, sizeof(buffer), stdin)) {
                if (buffer[strlen(buffer) - 1] == '\n') break;
            }
            continue;
        }
        buffer[length - 1] = '\0';
        for (vname = buffer; *vname == ' ' || *vname == '\t'; vname++);
        if (*vname == '#' || *vname == '\0') continue;
        if (!(value = strchr(vname, '='))) {
            exit(1);
            continue;
        }
        for (end = value - 1; *end == ' ' || *end == '\t'; end--);
        end[1] = '\0';
        for (p = blockdb; p->op_name; p++) {
            if (strcmp(p->op_name, vname) == 0) break;
        }
        if (!p->op_name) {
            printf("Operation unknown: %s\n", vname);
            exit(1);
        }
        for (value++; *value == ' ' || *value == '\t'; value++);
        for (end = buffer + length - 2; *end == ' ' || *end == '\t'; end--);
        end++; *end = '\0';

        // Always an integer...
        intvalue = strtol(value, &errorptr, 0);
        if (*errorptr != '\0') {
             printf("Invalid: %s\n", vname);
             exit(1);
         }
         *((int *) p->op_variable) = intvalue;

        int *p1=&v1k, *p2=&v2k, *p3=&v4k, *p4=&v8k;
        int *p5=&v16k, *p6=&v32k, *p7=&v64k, *p8=&v128k;

        //Read or Write...
        if (temp = strchr(p->op_name, 'r')) {
         coord = 23;
         gp = prvalues;
         gpt = ptrvalues;
        } else {
         coord= 47;
         gp = pwvalues;
         gpt = ptwvalues;
        }

        // Draw...
        if ((*p->op_variable > *p1) && (*p->op_variable <= *p2))
        {
           gp[1] = gp[1] + 1;
           gpt[1] = (gpt[1] + (*p->op_variable));
           print_int_middle(win, coord,      6, 5, &gp[1], COLOR_PAIR(0));
           print_int_middle(win, coord + 12, 6, 5, &gpt[1], COLOR_PAIR(0));
        } else if ((*p->op_variable > *p2) && (*p->op_variable <= *p3)) 
          {
                gp[2] = gp[2] + 1;
                gpt[2] = (gpt[2] + (*p->op_variable));
                print_int_middle(win, coord,      7, 5, &gp[2], COLOR_PAIR(0));
                print_int_middle(win, coord + 12, 7, 5, &gpt[2], COLOR_PAIR(0));
          } else if ((*p->op_variable > *p3) && (*p->op_variable <= *p4)) 
            {
                   gp[3] = gp[3] + 1;
                   gpt[3] = (gpt[3] + (*p->op_variable));
                   print_int_middle(win, coord,      8, 5, &gp[3], COLOR_PAIR(0));
                   print_int_middle(win, coord + 12, 8, 5, &gpt[3], COLOR_PAIR(0));
            } else if ((*p->op_variable > *p4) && (*p->op_variable <= *p5)) 
              {
                    gp[4] = gp[4] + 1;
                    gpt[4] = (gpt[4] + (*p->op_variable));
                    print_int_middle(win, coord,      9, 5, &gp[4], COLOR_PAIR(0));
                    print_int_middle(win, coord + 12, 9, 5, &gpt[4], COLOR_PAIR(0));
              } else if ((*p->op_variable > *p5) && (*p->op_variable <= *p6)) 
                {
                      gp[5] = gp[5] + 1;
                      gpt[5] = (gpt[5] + (*p->op_variable));
                      print_int_middle(win, coord,      10, 5, &gp[5], COLOR_PAIR(0));
                      print_int_middle(win, coord + 12, 10, 5, &gpt[5], COLOR_PAIR(0));
                } else if ((*p->op_variable > *p6) && (*p->op_variable <= *p7))
                  {
                        gp[6] = gp[6] + 1;
                        gpt[6] = (gpt[6] + (*p->op_variable));
                        print_int_middle(win, coord,      11, 5, &gp[6], COLOR_PAIR(0));
                        print_int_middle(win, coord + 12, 11, 5, &gpt[6], COLOR_PAIR(0));
                  } else if ((*p->op_variable > *p7) && (*p->op_variable <= *p8))
                    {
                          gp[7] = gp[7] + 1;
                          gpt[7] = (gpt[7] + *p->op_variable);
                          print_int_middle(win, coord,      12, 5, &gp[7], COLOR_PAIR(0));
                          print_int_middle(win, coord + 12, 12, 5, &gpt[7], COLOR_PAIR(0));
                    } else if (*p->op_variable > *p8)
                      {
                              gp[8] = gp[8] + 1;
                              gpt[8] = (gpt[8] + *p->op_variable);
                              print_int_middle(win, coord,      13, 5, &gp[8], COLOR_PAIR(0));
                              print_int_middle(win, coord + 12, 13, 5, &gpt[8], COLOR_PAIR(0));
                        } else {
                                gp[0] = gp[0] + 1;
                                gpt[0] = (gpt[0] + *p->op_variable);
                                print_int_middle(win, coord,      5, 5, &gp[0], COLOR_PAIR(0));
                                print_int_middle(win, coord + 12, 5, 5, &gpt[0], COLOR_PAIR(0));
        }
        wrefresh(win);
    }
    exit(0);
}

// ----------------= The main function =-----------------
int main (int argc, const char **argv) {
 struct dtraceoutput *pdtrace;
 int i, z=0, c; 
 MENU *menu_principal;
 ITEM **itens_menu;
 WINDOW *menu_principal_win;

 // Blocks and values array
 itens_menu = (ITEM **)calloc(IMENU, sizeof(ITEM *));

 for (i = 0; i < IMENU; i++)
 {
   itens_menu[i] = new_item(blocksizes[i], blocksizes[i]);
 }

 // Formatacao da tela... inicio do ncurses. 
 initscr();
 start_color();
 cbreak();
 noecho();
 keypad(stdscr, TRUE);
 init_pair(1, COLOR_RED, COLOR_BLACK);
 init_pair(2, COLOR_CYAN, COLOR_BLACK);
 init_pair(3, COLOR_MAGENTA, COLOR_BLACK);
 
 // Criacao do Menu principal 
 menu_principal = new_menu((ITEM **)itens_menu);

 // Nao mostrar a descricao de cada opcao...
 menu_opts_off(menu_principal, O_SHOWDESC);

 // Criacao da janela a ser associada com o menu 
 menu_principal_win = newwin(14, 70, 4, 4);
 keypad(menu_principal_win, TRUE);

 // Janela principal e sub menu.
 set_menu_win(menu_principal, menu_principal_win);
 set_menu_sub(menu_principal, derwin(menu_principal_win, 11, 68, 3, 1));
 set_menu_format(menu_principal, 17, 1);

 // Cursor 
 //set_menu_mark(menu_principal, "+ ");

 // Borda e titulo do menu 
 print_str_middle(menu_principal_win, 3, 11, 40, "R", COLOR_PAIR(0));
 print_str_middle(menu_principal_win, 3, 23, 40, "TR", COLOR_PAIR(0));
 print_str_middle(menu_principal_win, 3, 36, 40, "W", COLOR_PAIR(0));
 print_str_middle(menu_principal_win, 3, 48, 40, "TW", COLOR_PAIR(0));
 print_str_middle(menu_principal_win, 1, 15, 40, "---= eALL - NFS Server Monitoring Tool v0.1 =---", COLOR_PAIR(2));
 mvwaddch(menu_principal_win, 2, 0, ACS_LTEE);
 mvwhline(menu_principal_win, 2, 1, ACS_HLINE, 69);
 mvwaddch(menu_principal_win, 2, 69, ACS_RTEE);
 
 // Menu 
 post_menu(menu_principal);

 attron(COLOR_PAIR(2));
 mvprintw(LINES - 7, 26, "Pressione: <Ctrl C> para SAIR");
 mvprintw(LINES - 5, 25, "- http://www.eALL.com.br/blog -");
 mvprintw(LINES - 4, 15, "NFS Read/Write Monitoring Tool (Powered by Dtrace)");
 attroff(COLOR_PAIR(2));
 refresh();
 wrefresh(menu_principal_win);

 //Go, and don't come back!
 read_draw_data(pdtrace, menu_principal_win);
}

